@@ -95,9 +95,9 @@ ci.auc.smooth.roc <- function(smooth.roc,
     aucs <- aucs * 100
   }
   attr(ci, "conf.level") <- conf.level
+  attr(ci, "method") <- "bootstrap"
   attr(ci, "boot.n") <- boot.n
   attr(ci, "boot.stratified") <- boot.stratified
-  attr(ci, "aucs") <- aucs
   attr(ci, "auc") <- oldauc
   class(ci) <- "ci.auc"
   return(ci)  
@@ -105,6 +105,7 @@ ci.auc.smooth.roc <- function(smooth.roc,
 
 ci.auc.roc <- function(roc,
                    conf.level = 0.95,
+                   method=c("delong", "bootstrap"),
                    boot.n = 2000,
                    boot.stratified = TRUE,
                    reuse.auc=TRUE,
@@ -128,6 +129,52 @@ ci.auc.roc <- function(roc,
       attr(roc$auc, "partial.auc") <- attr(roc$auc, "partial.auc") / 100
   }
 
+  # Check the method
+  if (missing(method) | is.null(method)) {
+    # determine method if missing
+    if (is.numeric(attr(roc$auc, "partial.auc")) && length(attr(roc$auc, "partial.auc") == 2)) {
+      # partial auc: go for bootstrap
+      method <- "bootstrap"
+    }
+    else if (class(roc) == "smooth.roc") {
+      # smoothing: bootstrap
+      method <- "bootstrap"
+    }
+    else {
+      method <- "delong"
+    }
+  }
+  else {
+    method <- match.arg(method, c("delong", "bootstrap"))
+    # delong NA to pAUC: warn + change
+    if (is.numeric(attr(roc$auc, "partial.auc")) && length(attr(roc$auc, "partial.auc") == 2) && method == "delong") {
+      warning("Using DeLong for partial AUC is not supported. Using bootstrap instead.")
+      method <- "bootstrap"
+    }
+    else if (class(roc) == "smooth.roc") {
+      warning("Using DeLong's test for smoothed ROCs is not supported. Using bootstrap instead.")
+      method <- "bootstrap"
+    }
+  }
+
+  if (method == "delong")
+    ci <- ci.auc.delong(roc, conf.level)
+  else
+    ci <- ci.auc.bootstrap(roc, conf.level, boot.n, boot.stratified, progress, ...)
+
+  if (percent) {
+    ci <- ci * 100
+  }
+  attr(ci, "conf.level") <- conf.level
+  attr(ci, "method") <- method
+  attr(ci, "boot.n") <- boot.n
+  attr(ci, "boot.stratified") <- boot.stratified
+  attr(ci, "auc") <- oldauc
+  class(ci) <- "ci.auc"
+  return(ci)
+}
+  
+ci.auc.bootstrap <- function(roc, conf.level, boot.n, boot.stratified, progress, ...) {
   if(class(progress) != "list")
     progress <- roc.utils.get.progress.bar(progress, title="AUC confidence interval", label="Bootstrap in progress...", ...)
 
@@ -144,18 +191,8 @@ ci.auc.roc <- function(roc,
   }
   # TODO: Maybe apply a correction (it's in the Tibshirani?) What do Carpenter-Bithell say about that?
   # Prepare the return value
-  ci <- quantile(aucs, c(0+(1-conf.level)/2, .5, 1-(1-conf.level)/2))
-  if (percent) {
-    ci <- ci * 100
-    aucs <- aucs * 100
-  }
-  attr(ci, "conf.level") <- conf.level
-  attr(ci, "boot.n") <- boot.n
-  attr(ci, "boot.stratified") <- boot.stratified
-  attr(ci, "aucs") <- aucs
-  attr(ci, "auc") <- oldauc
-  class(ci) <- "ci.auc"
-  return(ci)
+  return(quantile(aucs, c(0+(1-conf.level)/2, .5, 1-(1-conf.level)/2)))
+
 }
 
 # Returns an auc in a stratified manner
@@ -242,3 +279,31 @@ nonstratified.ci.smooth.auc <- function(roc, smooth.roc.call, auc.call) {
     return(NA)
   return(eval(auc.call))
 }
+
+
+ci.auc.delong <- function(roc, conf.level) {
+  YR <- roc$controls # = C2, n, YRj
+  XR <- roc$cases # = C1, m, XRi
+
+  n <- length(YR)
+  m <- length(XR)
+  mn <- m*n
+
+  # Compute Mann-Whitney statistics and deduce thetaR and thetaS
+  MWR <- sapply(1:n, function(j) sapply(1:m, function(i, j) MW.kernel(XR[i], YR[j]), j=j))
+
+  thetaR <- sum(MWR)/mn
+
+  # Delong-specific computations
+  VR10 <- sapply(1:m, function(i) {sum(MWR[i,])})/n
+  VR01 <- sapply(1:n, function(j) {sum(MWR[,j])})/m
+
+  S10 <- sum((VR10 - thetaR) * (VR10 - thetaR))/(m-1)
+  S01 <- sum((VR01 - thetaR) * (VR01 - thetaR))/(n-1)
+  S <- S10/m + S01/n
+  ci <- qnorm(c(0+(1-conf.level)/2, .5, 1-(1-conf.level)/2), mean = thetaR, sd = sqrt(S))
+  if (roc$direction == ">") {
+    ci <- 1 - ci
+  }
+  return(ci)
+}
